package java.security.cert;

import java.util.Collection;
import java.util.List;
import java.util.Iterator;
import java.io.InputStream;
import java.security.NoSuchProviderException;
import java.security.NoSuchAlgorithmException;
import sun.security.jca.GetInstance;
import java.security.Provider;

public class CertificateFactory
{
    private String type;
    private Provider provider;
    private CertificateFactorySpi certFacSpi;
    
    protected CertificateFactory(CertificateFactorySpi certFacSpi, Provider provider, String type) {
        this.certFacSpi = certFacSpi;
        this.provider = provider;
        this.type = type;
    }
    
    public static final CertificateFactory getInstance(String s) throws CertificateException {
        try {
            GetInstance.Instance instance = GetInstance.getInstance("CertificateFactory", CertificateFactorySpi.class, s);
            return new CertificateFactory((CertificateFactorySpi)instance.impl, instance.provider, s);
        }
        catch (NoSuchAlgorithmException ex) {
            throw new CertificateException(s + " not found", ex);
        }
    }
    
    public static final CertificateFactory getInstance(String s, String provider) throws CertificateException, NoSuchProviderException {
        try {
            GetInstance.Instance instance = GetInstance.getInstance("CertificateFactory", CertificateFactorySpi.class, s, provider);
            return new CertificateFactory((CertificateFactorySpi)instance.impl, instance.provider, s);
        }
        catch (NoSuchAlgorithmException ex) {
            throw new CertificateException(s + " not found", ex);
        }
    }
    
    public static final CertificateFactory getInstance(String s, Provider provider) throws CertificateException {
        try {
            GetInstance.Instance instance = GetInstance.getInstance("CertificateFactory", CertificateFactorySpi.class, s, provider);
            return new CertificateFactory((CertificateFactorySpi)instance.impl, instance.provider, s);
        }
        catch (NoSuchAlgorithmException ex) {
            throw new CertificateException(s + " not found", ex);
        }
    }
    
    public final Provider getProvider() {
        return this.provider;
    }
    
    public final String getType() {
        return this.type;
    }
    
    public final Certificate generateCertificate(InputStream inputStream) throws CertificateException {
        return this.certFacSpi.engineGenerateCertificate(inputStream);
    }
    
    public final Iterator<String> getCertPathEncodings() {
        return this.certFacSpi.engineGetCertPathEncodings();
    }
    
    public final CertPath generateCertPath(InputStream inputStream) throws CertificateException {
        return this.certFacSpi.engineGenerateCertPath(inputStream);
    }
    
    public final CertPath generateCertPath(InputStream inputStream, String s) throws CertificateException {
        return this.certFacSpi.engineGenerateCertPath(inputStream, s);
    }
    
    public final CertPath generateCertPath(List<? extends Certificate> list) throws CertificateException {
        return this.certFacSpi.engineGenerateCertPath(list);
    }
    
    public final Collection<? extends Certificate> generateCertificates(InputStream inputStream) throws CertificateException {
        return this.certFacSpi.engineGenerateCertificates(inputStream);
    }
    
    public final CRL generateCRL(InputStream inputStream) throws CRLException {
        return this.certFacSpi.engineGenerateCRL(inputStream);
    }
    
    public final Collection<? extends CRL> generateCRLs(InputStream inputStream) throws CRLException {
        return this.certFacSpi.engineGenerateCRLs(inputStream);
    }
}
